// Functions for easy use of KAmod module with MMA8653FC accelerometer.
// jk 07'2018

#include "kamod_mma8653fc.h"
#include "kamami_l496_conn.h"

#define TIMEOUT						1000
		
#define MMA8653FC_STATUS			0x00
#define MMA8653FC_OUT_X_MSB			0x01
#define MMA8653FC_OUT_X_LSB			0x02
#define MMA8653FC_OUT_Y_MSB			0x03
#define MMA8653FC_OUT_Y_LSB			0x04
#define MMA8653FC_OUT_Z_MSB			0x05
#define MMA8653FC_OUT_Z_LSB			0x06
#define MMA8653FC_WHO_AM_I			0x0d
#define MMA8653FC_XYZ_DATA_CFG		0x0e
#define MMA8653FC_PL_STATUS			0x10	// Landscape/Portrait orientation status
#define MMA8653FC_PL_CFG			0x11	// Landscape/Portrait Configuration 	
#define MMA8653FC_CTRL_REG1			0x2a
#define MMA8653FC_CTRL_REG2			0x2b
#define MMA8653FC_CTRL_REG3			0x2c
#define MMA8653FC_CTRL_REG4			0x2d	// Interrupt Enable register
#define MMA8653FC_CTRL_REG5			0x2e	// Interrupt Configuration register

#define	MMA8653FC_PL_CFG_PL_EN		(1u << 6)

#define MMA8653FC_PL_STATUS_BAFRO		1
#define MMA8653FC_PL_STATUS_BAFRO_FRONT	0
#define MMA8653FC_PL_STATUS_BAFRO_BACK	1
#define MMA8653FC_PL_STATUS_LAPO_BIT0	1
#define MMA8653FC_PL_STATUS_LAPO_MSK	(3 << MMA8653FC_PL_STATUS_LAPO_BIT0)
#define MMA8653FC_PL_STATUS_BAFRO_LO	0x40

#define MMA8653FC_CTRL_REG1_DR_BIT0	3	
#define MMA8653FC_CTRL_REG1_ACTIVE	1

#define MMA8653FC_CTRL_REG2_RST		(1u << 6)

#define MMA8653FC_INT_ASLP			(1u << 7)	// Auto-Sleep/Wake 
#define MMA8653FC_INT_LNDPTR		(1u << 4)	// Orientation (Landscape/Portrait 
#define MMA8653FC_INT_FF_MT			(1u << 2)	// Freefall/Motion 
#define MMA8653FC_INT_DRDY			(1u << 0)	// Data Ready
#define MMA8653FC_INT_ALL			0x95		

#define	MMA8653FC_ID				0x5A

#define	MMA8653FC_ADDR				0x1D

#define	I2C_MRW						0x80	// multiple read/write

static const float fullscale_max_vals[3] = {2.0, 4.0, 8.0};
static float fullscale_v = 2.0;

static void mma8653fc_write8b(uint8_t reg, uint8_t val)
{
	conn_i2c_write(CONN_I2C_KAMOD, MMA8653FC_ADDR, reg, 1, &val, 1);
}

static void mma8653fc_reset()
{
	mma8653fc_write8b(MMA8653FC_CTRL_REG2, MMA8653FC_CTRL_REG2_RST);
}

int32_t mma8653fc_init(uint8_t datarate, uint8_t fullscale)
{
	uint8_t tmp = 0;
	
	conn_i2c_init(CONN_I2C_KAMOD);
	
	if (conn_i2c_read(CONN_I2C_KAMOD, MMA8653FC_ADDR, MMA8653FC_WHO_AM_I, 1, &tmp, 1) != 0)
		return -1;
	if (tmp != MMA8653FC_ID)
		return -2;
	
	mma8653fc_reset();
	mma8653fc_write8b(MMA8653FC_CTRL_REG1, 0);
	conn_i2c_read(CONN_I2C_KAMOD, MMA8653FC_ADDR, MMA8653FC_CTRL_REG1, 1, &tmp, 1); // that is needed (serves as a delay) 
	if (fullscale > MMA8653FC_FULLSCALE_8G)
		return - 1;
	fullscale_v = fullscale_max_vals[fullscale];	
	mma8653fc_write8b(MMA8653FC_XYZ_DATA_CFG, fullscale);	
	mma8653fc_write8b(MMA8653FC_PL_CFG, MMA8653FC_PL_CFG_PL_EN); // enable portrait/landscape detection
	if (datarate & 0xf8)
		return -1;
	mma8653fc_write8b(MMA8653FC_CTRL_REG1, (datarate << MMA8653FC_CTRL_REG1_DR_BIT0) | MMA8653FC_CTRL_REG1_ACTIVE);

	return 0;
}

float mma8653fc_res_to_float(int16_t res)
{
	return fullscale_v * (float)res / MMA8653FC_FULLSCALE_VAL;
}

int32_t mma8653fc_read_xyz(struct d3_res16* res)
{	
	uint8_t r[6];
	if (conn_i2c_read(CONN_I2C_KAMOD, MMA8653FC_ADDR, MMA8653FC_OUT_X_MSB | I2C_MRW, 1, r, 6) != 0)
		return -1;
	res->x = (r[0] << 8) | r[1];
	res->y = (r[2] << 8) | r[3];
	res->z = (r[4] << 8) | r[5];
	return 0;
}

int16_t mma8653fc_read_x(void)
{
	uint8_t r[2];
	if (conn_i2c_read(CONN_I2C_KAMOD, MMA8653FC_ADDR, MMA8653FC_OUT_X_MSB | I2C_MRW, 1, r, 2) != 0)
		return 0;
	return (r[0] << 8) | r[1];
}

int16_t mma8653fc_read_y(void)
{
	uint8_t r[2];
	if (conn_i2c_read(CONN_I2C_KAMOD, MMA8653FC_ADDR, MMA8653FC_OUT_Y_MSB | I2C_MRW, 1, r, 2) != 0)
		return 0;
	return (r[0] << 8) | r[1];
}

int16_t mma8653fc_read_z(void)
{
	uint8_t r[2];
	if (conn_i2c_read(CONN_I2C_KAMOD, MMA8653FC_ADDR, MMA8653FC_OUT_Z_MSB | I2C_MRW, 1, r, 2) != 0)
		return 0;
	return (r[0] << 8) | r[1];
}


int32_t mma8653fc_readf_xyz(struct d3_resf* res)
{	
	uint8_t r[6];
	if (conn_i2c_read(CONN_I2C_KAMOD, MMA8653FC_ADDR, MMA8653FC_OUT_X_MSB | I2C_MRW, 1, r, 6) != 0)
		return -1;
	res->x = mma8653fc_res_to_float((r[0] << 8) | r[1]);
	res->y = mma8653fc_res_to_float((r[2] << 8) | r[3]);
	res->z = mma8653fc_res_to_float((r[4] << 8) | r[5]);
	return 0;
}

float mma8653fc_readf_x(void)
{
	uint8_t r[2];
	if (conn_i2c_read(CONN_I2C_KAMOD, MMA8653FC_ADDR, MMA8653FC_OUT_X_MSB | I2C_MRW, 1, r, 2) != 0)
		return 0;
	return mma8653fc_res_to_float((r[0] << 8) | r[1]);
}

float mma8653fc_readf_y(void)
{
	uint8_t r[2];
	if (conn_i2c_read(CONN_I2C_KAMOD, MMA8653FC_ADDR, MMA8653FC_OUT_Y_MSB | I2C_MRW, 1, r, 2) != 0)
		return 0;
	return mma8653fc_res_to_float((r[0] << 8) | r[1]);
}

float mma8653fc_readf_z(void)
{
	uint8_t r[2];
	if (conn_i2c_read(CONN_I2C_KAMOD, MMA8653FC_ADDR, MMA8653FC_OUT_Z_MSB | I2C_MRW, 1, r, 2) != 0)
		return 0;
	return mma8653fc_res_to_float((r[0] << 8) | r[1]);
}

static enum orientation_ otab[4] = {ORIENTATION_PORTRAIT_UP, ORIENTATION_PORTRAIT_DOWN, 
	ORIENTATION_LANDSCAPE_RIGHT, ORIENTATION_LANDSCAPE_LEFT};
static enum orientation_flat_ oftab[2] = {ORIENTATION_FLAT_FRONT, ORIENTATION_FLAT_BACK};

int32_t mma8653fc_read_orientation(struct orientation_res* res)
{
	uint8_t r;
	if (conn_i2c_read(CONN_I2C_KAMOD, MMA8653FC_ADDR, MMA8653FC_PL_STATUS, 1, &r, 1) != 0)
		return -1;
	res->orientation_flat = oftab[r & MMA8653FC_PL_STATUS_BAFRO];
	res->orientation = (r & MMA8653FC_PL_STATUS_BAFRO_LO) ? ORIENTATION_UNDETERMINED 
		: otab[(r & MMA8653FC_PL_STATUS_LAPO_MSK) >> MMA8653FC_PL_STATUS_LAPO_BIT0];
	return r;
}	

